package require stooop
namespace import stooop::*

namespace eval ::Aldec::NCsimConverter {

	#list of Argument object
	class Arguments {	
	
		proc Arguments { this _tokens } {
			set ($this,m_tokens) $_tokens
			
			set ($this,m_argumentsList) {}
			
			#read tokens until last token or first close-brace
			
			#append command name
			set token [::Aldec::NCsimConverter::Tokens::getCurrent $_tokens]
			set tokenStr [::Aldec::NCsimConverter::Token::getString $token]
			set firstIdx [::Aldec::NCsimConverter::Tokens::getIdx $_tokens]
			lappend ($this,m_argumentsList) [new ::Aldec::NCsimConverter::Argument $_tokens $tokenStr $firstIdx $firstIdx]			
			
			while { [::Aldec::NCsimConverter::Tokens::next $_tokens token] } {
				#omit empty and converted
				if { [::Aldec::NCsimConverter::Token::isConverted $token] \
						|| [::Aldec::NCsimConverter::Token::isEmpty $token] } {
					continue				
				}
				
				#close brace - end of analysis
				if { [::Aldec::NCsimConverter::Token::isCloseBrace $token] } {					
					break
				}
				
				#open brace - get braced body
				if { [::Aldec::NCsimConverter::Token::isOpenBrace $token] } {
					::Aldec::NCsimConverter::Tokens::prev $_tokens token
					set bodyStr [getBraceBody $this firstIdx lastIdx]
					lappend ($this,m_argumentsList) [new ::Aldec::NCsimConverter::Argument $_tokens $bodyStr $firstIdx $lastIdx]
					continue
				}
				
				#simple token
				set tokenStr [::Aldec::NCsimConverter::Token::getString $token]
				set firstIdx [::Aldec::NCsimConverter::Tokens::getIdx $_tokens]
				lappend ($this,m_argumentsList) [new ::Aldec::NCsimConverter::Argument $_tokens $tokenStr $firstIdx $firstIdx]
			}
			
			set ($this,m_idx) 0
			set ($this,m_maxIdx) [expr [llength $($this,m_argumentsList)] - 1]
		}
		
		proc ~Arguments { this } {
			eval delete $($this,m_argumentsList)
		}
		
		proc getTokensObject { this } {
			return $($this,m_tokens)
		}
		
		proc getBraceBody { this _firstIdx _lastIdx } {
			upvar $_firstIdx firstIdx
			upvar $_lastIdx lastIdx
			
			set firstIdx [::Aldec::NCsimConverter::Tokens::getIdx $($this,m_tokens)]
			
			set bodyStr ""
			while { [::Aldec::NCsimConverter::Tokens::next $($this,m_tokens) token] } {
				set str [::Aldec::NCsimConverter::Token::getString $token]
				append bodyStr $str 
				if { [::Aldec::NCsimConverter::Token::isCloseBrace $token] } {
					set lastIdx [::Aldec::NCsimConverter::Tokens::getIdx $($this,m_tokens)]					
					return $bodyStr
				}				
			}
			
			error "Cannot read braced argument."
		}
		
		#get second argument as string
		proc getSubcommandName { this } {
			if [getSubcommandAsArgument $this arg] {				
				return [::Aldec::NCsimConverter::Argument::getString $arg]			
			} else {
				return ""			
			}
		}
		
		#replace first argument string
		proc replaceCommandName { this _newValue } {
			getCommandAsArgument $this arg
			::Aldec::NCsimConverter::Argument::setString $arg $_newValue
			::Aldec::NCsimConverter::Argument::setConverted $arg						
		}		
		
		#replace second argument string
		proc replaceSubcommandName { this _newValue } {
			if [getSubcommandAsArgument $this arg] {
				::Aldec::NCsimConverter::Argument::setString $arg $_newValue
				::Aldec::NCsimConverter::Argument::setConverted $arg
			}		
		}		
		
		#get second argument from list
		proc getSubcommandAsArgument { this _arg } {
			if { [llength $($this,m_argumentsList)] < 2 } { return 0 }

			upvar $_arg arg
			set arg [lindex $($this,m_argumentsList) 1]
			return 1
		}
		
		#get first argument from list
		proc getCommandAsArgument { this _arg } {
			if { [llength $($this,m_argumentsList)] < 1 } { return 0 }

			upvar $_arg arg
			set arg [lindex $($this,m_argumentsList) 0]
			return 1
		}		

		#returns 1 and move iterator to next not converted item		
		proc nextArgument { this _outArg } {
			upvar $_outArg outArg
			
			while { true } {
				if { $($this,m_idx) >= $($this,m_maxIdx) } {
					return 0			
				}
				
				incr ($this,m_idx)
				set outArg [lindex $($this,m_argumentsList) $($this,m_idx)]
				if { ![::Aldec::NCsimConverter::Argument::isConverted $outArg] } {
					return 1				
				}			
			}
			
			return 0
		}
		
		
		proc getInternalArgumentsList { this _list _curIdx _maxIdx } {
			upvar $_list list
			upvar $_curIdx curIdx
			upvar $_maxIdx maxIdx
			set list $($this,m_argumentsList)
			set curIdx $($this,m_idx)
			set maxIdx $($this,m_maxIdx)
		}
		
		#clear all arguments strings and tokens between arguments
		proc clearRange { this _firstArgIdx _lastArgIdx } {
		
			::Aldec::NCsimConverter::Argument::getTokensIdx [lindex $($this,m_argumentsList) $_firstArgIdx] firstTokenIdx tmp
			::Aldec::NCsimConverter::Argument::getTokensIdx [lindex $($this,m_argumentsList) $_lastArgIdx] tmp lastTokenIdx
			
			for {} { $firstTokenIdx <= $lastTokenIdx } { incr firstTokenIdx } {
				set token [::Aldec::NCsimConverter::Tokens::getToken $($this,m_tokens) $firstTokenIdx]
				::Aldec::NCsimConverter::Token::setString $token ""				
			}
			
			for {} { $_firstArgIdx <= $_lastArgIdx } { incr _firstArgIdx } {
				set arg [lindex $($this,m_argumentsList) $_firstArgIdx]
				::Aldec::NCsimConverter::Argument::setString $arg ""			
			}
		}
		
		#check if any argument is equal to _switchName
		proc existsSwitch { this _switchName } {
		
			foreach arg $($this,m_argumentsList) {
				if { [::Aldec::NCsimConverter::Argument::getString $arg] == $_switchName } {
					return 1
				}
			}	
			return 0
		}
		
		#returns idx: -1 if not found
		proc findSwitchIdx { this _switchName } {
			set idx 0
			foreach arg $($this,m_argumentsList) {
				if { [::Aldec::NCsimConverter::Argument::getString $arg] == $_switchName } {
					return $idx
				}
				incr idx
			}	
			return -1
		}
	
		#without command name!		
		proc matchFirstArguments { this _listOfArguments } {
		
			set i 1
			foreach argument $_listOfArguments {
				if { $i > $($this,m_maxIdx) } {
					return 0
				}
				if { $argument != [::Aldec::NCsimConverter::Argument::getString [lindex $($this,m_argumentsList) $i]] } {
					return 0
				}
				incr i			
			}
			
			return 1					
		}
		
		proc getCurrentArg { this } {
			return [lindex $($this,m_argumentsList) $($this,m_idx)]		
		}
		
		proc getArgByIdx { this idx } {
			return [lindex $($this,m_argumentsList) $idx]		
		}
		
		proc appendCommands { this commandLine } {
			set arg [lindex $($this,m_argumentsList) end]
			set str [::Aldec::NCsimConverter::Argument::getString $arg]
			::Aldec::NCsimConverter::Argument::setString $arg $str$commandLine
		}
	}
}
