namespace eval ::Aldec::NCsimConverter::Replacers {

	namespace export \
		replaceSwitch \
		replaceSwitchUsingFunction \
		replaceWholeCommand
		
		
	proc replaceSwitchUsingFunction { _args _patterns _callback } {
	
#add wave -divider
#add wave -divider ""
#add wave -divider {}
#add wave -divider " -Example- "
#add wave -divider " -Example- " test other_divider another_div
#add wave -divider Example -hex
#add wave -divider " -Example- " test other_divider another_div -hex		
	
		::Aldec::NCsimConverter::Arguments::getInternalArgumentsList $_args argsList firstIdx maxIdx
		
		set curIdx $firstIdx
	
		set arg [::Aldec::NCsimConverter::Arguments::getCurrentArg $_args]
		
		if { ![::Aldec::NCsimConverter::Argument::isConverted $arg] } {
		
		}
	}
	
	#- replaces switch with arguments to another value
	#- it recognizes expressions <string%0> .. <string%n>
	#- it accepts only space-separated list as _patterns: '-l <string%0> <string%1>'
	#		however '_newValue' can be freely constructed: '-o<string%1>+<string%0>'
	#- it accepts regular expressions: <regexp%<id>%<regular expression>>
	#	
	proc replaceSwitch { _args _patterns _newValue } {
	
		::Aldec::NCsimConverter::Arguments::getInternalArgumentsList $_args argsList firstIdx maxIdx
		
		set curIdx $firstIdx
	
		set arg [::Aldec::NCsimConverter::Arguments::getCurrentArg $_args]
		
		if { ![::Aldec::NCsimConverter::Argument::isConverted $arg] } {

			#try to match parameters from '_patterns' and from 'argsList'
			for { set i 0 } { $i <= [expr [llength $_patterns] - 1] && $curIdx <= $maxIdx } { incr i; incr curIdx } {
			
				set pattern [lindex $_patterns $i]				
				set curArg [lindex $argsList $curIdx]
				set argValue [::Aldec::NCsimConverter::Argument::getString $curArg]
				
				if { [isExpressionInside $pattern] } {

					splitPatternToExpression $pattern prefix expression
					
					if { ![string match "${prefix}*" $argValue] } { 
						#argument doesn't match requested pattern
						return
					}
					
					#postfix should be matched with requested expression
					set postfix [string range $argValue [string length $prefix] end]
					
					set expressionType [getExpressionType $expression]
					
					if { $expressionType == [EXPRESSION_STRING] || $expressionType == [EXPRESSION_FILE] } {
						#exchange expressions like <string%0> inside _newValue
						regsub -all -- "$expression" $_newValue $postfix _newValue
					} elseif { $expressionType == [EXPRESSION_REGEXP] } {
						set regExp [getRegexpFromExpression $expression]
						
						#convert expression name
						regexp {^<regexp%([0-9]+)%.+>$} $expression tmp parameterID
						set properName "<regexp%${parameterID}>"
						
						if { ![regexp $regExp $postfix] } {
							#argument doesn't match regexp
							return
						}
						
						#exchange expressions like <regexp%0> inside _newValue
						regsub -all -- "$properName" $_newValue $postfix _newValue
					
					} else {
						error "Not handled expression type!"
					}					
				
				} else {
					#match strings 1:1
					if { [::Aldec::NCsimConverter::Argument::getString $curArg] != $pattern } {
						#arguments don't match
						return 
					}
				}
			}
			
			#not every patterns was analyzed (matched)
			if { $i != [llength $_patterns] } {
				return
			}
			
			::Aldec::NCsimConverter::Arguments::clearRange $_args $firstIdx [expr $curIdx - 1]
			
			#set value onlu for first argument object
			::Aldec::NCsimConverter::Argument::setString [lindex $argsList $firstIdx] $_newValue
		}
	}
	
	#replaces whole currently analyzed command (and all arguments) with '_newValue'
	# 
	proc replaceWholeCommand { _args _newValue } {
		::Aldec::NCsimConverter::Arguments::getInternalArgumentsList $_args argsList curIdx maxIdx

		foreach arg $argsList {
			::Aldec::NCsimConverter::Argument::setString $arg ""
		}

		::Aldec::NCsimConverter::Argument::setString [lindex $argsList 0] $_newValue
	}
	
	#----------------------------- private -----------------------------------
	proc getExpressionType { _name } {
		if { [regexp {^<string%[0-9]+>$} $_name] } { return [EXPRESSION_STRING] }
		if { [regexp {^<file%[0-9]+>$} $_name] } { return [EXPRESSION_FILE] }
		if { [regexp {^<regexp%[0-9]+%.+>$} $_name] } { return [EXPRESSION_REGEXP] }
		return [EXPRESSION_UNKNOWN]
	}
	
	proc getRegexpFromExpression { _expression } {
		set _regExp ""
		regexp {^<regexp%[0-9]+%(.+)>$} $_expression tmp _regExp
		return $_regExp
	}
	
	#splits patterns: '<any_string><any_expression' for example: '+acc=<string%0>' -> '+acc=' and '<string%0>'
	proc splitPatternToExpression { _patterns _prefix _expression } {
		upvar $_prefix prefix
		upvar $_expression expression
		return [regexp -- {^(.*)(<.*>)$} $_patterns tmp prefix expression]
	}
	
	proc isExpressionInside { _patterns } {
		set expression ""
		if { ![regexp {(<.*>)} $_patterns tmp expression] } {
			return 0
		}
		
		return [expr [getExpressionType $expression] != [EXPRESSION_UNKNOWN]]
	}
	
	proc EXPRESSION_UNKNOWN {} { return 0 }
	proc EXPRESSION_STRING {} { return 1 }
	proc EXPRESSION_FILE {} { return 2 }
	proc EXPRESSION_REGEXP {} { return 3 }
}